@@ -15,7 +15,7 @@
 # THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 # OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABIL-
 # ITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT
-# SHALL THE AUTHOR BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, 
+# SHALL THE AUTHOR BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 # WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 # OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 # IN THE SOFTWARE.
@@ -29,7 +29,9 @@ import boto
 import boto.s3.key
 from botoweb.db.blob import Blob
 import logging
-log = logging.getLogger("newscore")
+log = logging.getLogger('newscore')
+
+
 class Property(object):
 
 	data_type = str
@@ -51,9 +53,9 @@ class Property(object):
 		else:
 			self.slot_name = '_'
 		self.unique = unique
-		
+
 	def __get__(self, obj, objtype):
-		#log.info("%s,%s" % (self.slot_name,objtype))
+		#log.info('%s,%s' % (self.slot_name,objtype))
 		#log.info(self.verbose_name)
 		if obj:
 			obj.load()
@@ -67,11 +69,11 @@ class Property(object):
 
 		# Fire off any on_set functions
 		try:
-			if obj._loaded and hasattr(obj, "on_set_%s" % self.name):
-				fnc = getattr(obj, "on_set_%s" % self.name)
+			if obj._loaded and hasattr(obj, 'on_set_%s' % self.name):
+				fnc = getattr(obj, 'on_set_%s' % self.name)
 				value = fnc(value)
 		except Exception:
-			boto.log.exception("Exception running on_set_%s" % self.name)
+			boto.log.exception('Exception running on_set_%s' % self.name)
 
 		setattr(obj, self.slot_name, value)
 
@@ -84,16 +86,16 @@ class Property(object):
 		if isinstance(value, basestring) or value == self.default_value():
 			return
 		if not isinstance(value, self.data_type):
-			raise TypeError, 'Validation Error, %s.%s expecting %s, got %s' % (self.model_class.__name__, self.name, self.data_type, type(value))
-									  
+			raise TypeError('Validation Error, %s.%s expecting %s, got %s' % (self.model_class.__name__, self.name, self.data_type, type(value)))
+
 	def default_value(self):
 		return self.default
 
 	def validate(self, value):
-		if self.required and value==None:
-			raise ValueError, '%s is a required property' % self.name
+		if self.required and value is None:
+			raise ValueError('%s is a required property' % self.name)
 		if self.choices and value and not value in self.choices:
-			raise ValueError, '%s not a valid choice for %s.%s' % (value, self.model_class.__name__, self.name)
+			raise ValueError('%s not a valid choice for %s.%s' % (value, self.model_class.__name__, self.name))
 		if self.validator:
 			self.validator(value)
 		else:
@@ -114,27 +116,30 @@ class Property(object):
 			return self.choices()
 		return self.choices
 
+
 def validate_string(value):
-	if value == None:
+	if value is None:
 		return
 	elif isinstance(value, str) or isinstance(value, unicode):
 		if len(value) > 1024:
-			raise ValueError, 'Length of value greater than maxlength'
+			raise ValueError('Length of value greater than maxlength')
 	else:
-		raise TypeError, 'Expecting String, got %s' % type(value)
+		raise TypeError('Expecting String, got %s' % type(value))
+
 
 class StringProperty(Property):
 
 	type_name = 'String'
-	
+
 	def __init__(self, verbose_name=None, name=None, default='', required=False,
 				validator=validate_string, choices=None, unique=False):
 		Property.__init__(self, verbose_name, name, default, required, validator, choices, unique)
 
+
 class TextProperty(Property):
-	
+
 	type_name = 'Text'
-	
+
 	def __init__(self, verbose_name=None, name=None, default='', required=False,
 				validator=None, choices=None, unique=False, max_length=None):
 		Property.__init__(self, verbose_name, name, default, required, validator, choices, unique)
@@ -143,12 +148,13 @@ class TextProperty(Property):
 	def validate(self, value):
 		value = super(TextProperty, self).validate(value)
 		if not isinstance(value, str) and not isinstance(value, unicode):
-			raise TypeError, 'Expecting Text, got %s' % type(value)
+			raise TypeError('Expecting Text, got %s' % type(value))
 		if self.max_length and len(value) > self.max_length:
-			raise ValueError, 'Length of value greater than maxlength %s' % self.max_length
+			raise ValueError('Length of value greater than maxlength %s' % self.max_length)
+
 
 class PasswordProperty(StringProperty):
-	"""
+	'''
 
 	Hashed property whose original value can not be
 	retrieved, but still can be compared.
@@ -159,7 +165,7 @@ class PasswordProperty(StringProperty):
 
 	The comparison
 
-	   obj.password == 'foo' 
+	   obj.password == 'foo'
 
 	generates a hash of 'foo' and compares it to the
 	stored hash.
@@ -170,7 +176,7 @@ class PasswordProperty(StringProperty):
 	where sha512 is not available )
 
 	It's unlikely you'll ever need to use a different hash
-	function, but if you do, you can control the behavior 
+	function, but if you do, you can control the behavior
 	in one of two ways:
 
 	  1) Specifying hashfunc in PasswordProperty constructor
@@ -181,32 +187,31 @@ class PasswordProperty(StringProperty):
 			 password = PasswordProperty(hashfunc=hashlib.sha224)
 
 	  2) Subclassing Password and PasswordProperty
-		
+
 		 class SHA224Password(Password):
 			 hashfunc=hashlib.sha224
 
 		 class SHA224PasswordProperty(PasswordProperty):
 			 data_type=MyPassword
-			 type_name="MyPassword"
+			 type_name='MyPassword'
 
 		 class MyModel(Model):
 			 password = SHA224PasswordProperty()
 
-	"""
+	'''
 	data_type = Password
 	type_name = 'Password'
 
 	def __init__(self, verbose_name=None, name=None, default='', required=False,
 				validator=None, choices=None, unique=False, hashfunc=None):
 
-		"""
+		'''
 		   The hashfunc parameter overrides the default hashfunc in boto.utils.Password.
 
-		   The remaining parameters are passed through to StringProperty.__init__"""
-
+		   The remaining parameters are passed through to StringProperty.__init__'''
 
 		StringProperty.__init__(self, verbose_name, name, default, required, validator, choices, unique)
-		self.hashfunc=hashfunc
+		self.hashfunc = hashfunc
 
 	def make_value_from_datastore(self, value):
 		p = self.data_type(value, hashfunc=self.hashfunc)
@@ -233,13 +238,14 @@ class PasswordProperty(StringProperty):
 		value = Property.validate(self, value)
 		if isinstance(value, self.data_type):
 			if len(value) > 1024:
-				raise ValueError, 'Length of value greater than maxlength'
+				raise ValueError('Length of value greater than maxlength')
 		else:
-			raise TypeError, 'Expecting %s, got %s' % (type(self.data_type), type(value))
+			raise TypeError('Expecting %s, got %s' % (type(self.data_type), type(value)))
+
 
 class BlobProperty(Property):
 	data_type = Blob
-	type_name = "blob"
+	type_name = 'blob'
 
 	def __set__(self, obj, value):
 		if value != self.default_value():
@@ -252,11 +258,12 @@ class BlobProperty(Property):
 				value = b
 		Property.__set__(self, obj, value)
 
+
 class S3KeyProperty(Property):
-	
+
 	data_type = boto.s3.key.Key
 	type_name = 'S3Key'
-	validate_regex = "^s3:\/\/([^\/]*)\/(.*)$"
+	validate_regex = '^s3:\/\/([^\/]*)\/(.*)$'
 
 	def __init__(self, verbose_name=None, name=None, default=None,
 				required=False, validator=None, choices=None, unique=False):
@@ -272,7 +279,7 @@ class S3KeyProperty(Property):
 		match = re.match(self.validate_regex, value)
 		if match:
 			return
-		raise TypeError, 'Validation Error, expecting %s, got %s' % (self.data_type, type(value))
+		raise TypeError('Validation Error, expecting %s, got %s' % (self.data_type, type(value)))
 
 	def __get__(self, obj, objtype):
 		value = Property.__get__(self, obj, objtype)
@@ -286,18 +293,19 @@ class S3KeyProperty(Property):
 				k = bucket.get_key(match.group(2))
 				if not k:
 					k = bucket.new_key(match.group(2))
-					k.set_contents_from_string("")
+					k.set_contents_from_string('')
 				return k
 		else:
 			return value
-		
+
 	def get_value_for_datastore(self, model_instance):
 		value = Property.get_value_for_datastore(self, model_instance)
 		if value:
-			return "s3://%s/%s" % (value.bucket.name, value.name)
+			return 's3://%s/%s' % (value.bucket.name, value.name)
 		else:
 			return None
 
+
 class IntegerProperty(Property):
 
 	data_type = int
@@ -313,21 +321,20 @@ class IntegerProperty(Property):
 		value = int(value)
 		value = Property.validate(self, value)
 		if value > self.max:
-			raise ValueError, 'Maximum value is %d' % self.max
+			raise ValueError('Maximum value is %d' % self.max)
 		if value < self.min:
-			raise ValueError, 'Minimum value is %d' % self.min
+			raise ValueError('Minimum value is %d' % self.min)
 		return value
-	
+
 	def empty(self, value):
 		return value is None
 
 	def __set__(self, obj, value):
-		if value == "" or value == None:
+		if value == '' or value is None:
 			value = 0
 		return Property.__set__(self, obj, value)
 
 
-
 class LongProperty(Property):
 
 	data_type = long
@@ -343,11 +350,11 @@ class LongProperty(Property):
 		min = -9223372036854775808
 		max = 9223372036854775807
 		if value > max:
-			raise ValueError, 'Maximum value is %d' % max
+			raise ValueError('Maximum value is %d' % max)
 		if value < min:
-			raise ValueError, 'Minimum value is %d' % min
+			raise ValueError('Minimum value is %d' % min)
 		return value
-		
+
 	def empty(self, value):
 		return value is None
 
@@ -362,7 +369,7 @@ class BooleanProperty(Property):
 
 	def empty(self, value):
 		return value is None
-	
+
 class FloatProperty(Property):
 
 	data_type = float
@@ -376,14 +383,14 @@ class FloatProperty(Property):
 		value = float(value)
 		value = Property.validate(self, value)
 		return value
-	
+
 	def empty(self, value):
 		return value is None
 
 class DateTimeProperty(Property):
-	"""This class handles both the datetime.datetime object
+	'''This class handles both the datetime.datetime object
 	And the datetime.date objects. It can return either one,
-	depending on the value stored in the database"""
+	depending on the value stored in the database'''
 
 	data_type = datetime.datetime
 	type_name = 'DateTime'
@@ -400,7 +407,7 @@ class DateTimeProperty(Property):
 		return Property.default_value(self)
 
 	def validate(self, value):
-		if value == None:
+		if value is None:
 			return
 		if isinstance(value, datetime.date):
 			return value
@@ -432,10 +439,10 @@ class DateProperty(Property):
 
 	def validate(self, value):
 		value = super(DateProperty, self).validate(value)
-		if value == None:
+		if value is None:
 			return
 		if not isinstance(value, self.data_type):
-			raise TypeError, 'Validation Error, expecting %s, got %s' % (self.data_type, type(value))
+			raise TypeError('Validation Error, expecting %s, got %s' % (self.data_type, type(value)))
 
 	def get_value_for_datastore(self, model_instance):
 		if self.auto_now:
@@ -462,7 +469,7 @@ class TimeProperty(Property):
 		if value is None:
 			return
 		if not isinstance(value, self.data_type):
-			raise TypeError, 'Validation Error, expecting %s, got %s' % (self.data_type, type(value))
+			raise TypeError('Validation Error, expecting %s, got %s' % (self.data_type, type(value)))
 
 
 class ReferenceProperty(Property):
@@ -475,7 +482,7 @@ class ReferenceProperty(Property):
 		Property.__init__(self, verbose_name, name, default, required, validator, choices, unique)
 		self.reference_class = reference_class
 		self.collection_name = collection_name
-		
+
 	def __get__(self, obj, objtype):
 		if obj:
 			value = getattr(obj, self.slot_name)
@@ -490,10 +497,10 @@ class ReferenceProperty(Property):
 			return value
 
 	def __set__(self, obj, value):
-		"""Don't allow this object to be associated to itself
-		This causes bad things to happen"""
-		if value != None and (obj.id == value or (hasattr(value, "id") and obj.id == value.id)):
-			raise ValueError, "Can not associate an object with itself!"
+		'''Don't allow this object to be associated to itself
+		This causes bad things to happen'''
+		if value is not None and (obj.id == value or (hasattr(value, 'id') and obj.id == value.id)):
+			raise ValueError('Can not associate an object with itself!')
 		return super(ReferenceProperty, self).__set__(obj,value)
 
 	def __property_config__(self, model_class, property_name):
@@ -501,12 +508,12 @@ class ReferenceProperty(Property):
 		if self.collection_name is None:
 			self.collection_name = '%s_%s_set' % (model_class.__name__.lower(), self.name)
 		if hasattr(self.reference_class, self.collection_name):
-			raise ValueError, 'duplicate property: %s' % self.collection_name
+			raise ValueError('duplicate property: %s' % self.collection_name)
 		setattr(self.reference_class, self.collection_name,
 				_ReverseReferenceProperty(model_class, property_name, self.collection_name))
 
 	def check_uuid(self, value):
-		# This does a bit of hand waving to "type check" the string
+		# This does a bit of hand waving to 'type check' the string
 		t = value.split('-')
 		if len(t) != 5:
 			raise ValueError
@@ -518,14 +525,14 @@ class ReferenceProperty(Property):
 			if obj_lineage.startswith(cls_lineage):
 				return
 		except:
-			raise ValueError, '%s is not a Model' % repr(value)
-		raise TypeError, '%s not instance of %s' % (obj_lineage, cls_lineage)
-			
+			raise ValueError('%s is not a Model' % repr(value))
+		raise TypeError('%s not instance of %s' % (obj_lineage, cls_lineage))
+
 	def validate(self, value):
 		if self.validator:
 			self.validator(value)
 		if self.required and value==None:
-			raise ValueError, '%s is a required property' % self.name
+			raise ValueError('%s is a required property' % self.name)
 		if value == self.default_value():
 			return
 		if not isinstance(value, str) and not isinstance(value, unicode):
@@ -543,27 +550,15 @@ class _ReverseReferenceProperty(Property):
 		self.item_type = model
 
 	def __get__(self, model_instance, model_class):
-		"""Fetches collection of model instances of this collection property."""
-		from botoweb.db.dynamo import DynamoModel
+		'''Fetches collection of model instances of this collection property.'''
 		if model_instance is not None:
-			# Support DynamoDB Objects
-			if DynamoModel in self.__model.mro():
-				return self.__model.search(bq="%s:'%s'" % (self.__property, model_instance.id))
-			else:
-				query = Query(self.__model)
-				if type(self.__property) == list:
-					props = []
-					for prop in self.__property:
-						props.append("%s =" % prop)
-					return query.filter(props, model_instance)
-				else:
-					return query.filter(self.__property + ' =', model_instance)
+			return self.__model.match_reference_property(self.__property, model_instance)
 		else:
 			return self
 
 	def __set__(self, model_instance, value):
-		"""Not possible to set a new collection."""
-		raise ValueError, 'Virtual property is read-only'
+		'''Not possible to set a new collection.'''
+		raise ValueError('Virtual property is read-only')
 
 
 class CalculatedProperty(Property):
@@ -576,7 +571,7 @@ class CalculatedProperty(Property):
 							validator, choices, unique)
 		self.calculated_type = calculated_type
 		self.use_method = use_method
-		
+
 	def __get__(self, obj, objtype):
 		value = self.default_value()
 		if obj:
@@ -589,7 +584,7 @@ class CalculatedProperty(Property):
 		return value
 
 	def __set__(self, obj, value):
-		"""Not possible to set a new AutoID."""
+		'''Not possible to set a new AutoID.'''
 		pass
 
 	def _set_direct(self, obj, value):
@@ -604,7 +599,7 @@ class CalculatedProperty(Property):
 			return None
 
 class ListProperty(Property):
-	
+
 	data_type = list
 	type_name = 'List'
 
@@ -631,7 +626,7 @@ class ListProperty(Property):
 		for item in value:
 			if not isinstance(item, item_type):
 				if item_type == (int, long):
-					raise ValueError, 'Items in the %s list must all be integers.' % self.name
+					raise ValueError('Items in the %s list must all be integers.' % self.name)
 				else:
 					raise ValueError('Items in the %s list must all be %s instances, got a %s instead' %
 									(self.name, self.item_type.__name__, type(item)))
@@ -644,7 +639,7 @@ class ListProperty(Property):
 		return list(super(ListProperty, self).default_value())
 
 	def __set__(self, obj, value):
-		"""Override the set method to allow them to set the property to an instance of the item_type instead of requiring a list to be passed in"""
+		'''Override the set method to allow them to set the property to an instance of the item_type instead of requiring a list to be passed in'''
 		if self.item_type in (int, long):
 			item_type = (int, long)
 		elif self.item_type in (str, unicode):
@@ -653,13 +648,68 @@ class ListProperty(Property):
 			item_type = self.item_type
 		if isinstance(value, item_type):
 			value = [value]
-		elif value == None: # Override to allow them to set this to "None" to remove everything
+		elif value is None:  # Override to allow them to set this to 'None' to remove everything
 			value = []
 		return super(ListProperty, self).__set__(obj,value)
 
 
+class SetProperty(Property):
+
+	data_type = set
+	type_name = 'Set'
+
+	def __init__(self, item_type, verbose_name=None, name=None, default=None, **kwds):
+		if default is None:
+			default = set()
+		self.item_type = item_type
+		Property.__init__(self, verbose_name, name, default=default, required=True, **kwds)
+
+	def validate(self, value):
+		if self.validator:
+			self.validator(value)
+		if value is not None:
+			if not isinstance(value, set):
+				value = set(value)
+
+		if self.item_type in (int, long):
+			item_type = (int, long)
+		elif self.item_type in (str, unicode):
+			item_type = (str, unicode)
+		else:
+			item_type = self.item_type
+
+		for item in value:
+			if not isinstance(item, item_type):
+				if item_type == (int, long):
+					raise ValueError('Items in the %s list must all be integers.' % self.name)
+				else:
+					raise ValueError('Items in the %s list must all be %s instances, got a %s instead' %
+									(self.name, self.item_type.__name__, type(item)))
+		return value
+
+	def empty(self, value):
+		return value is None
+
+	def default_value(self):
+		return set(super(SetProperty, self).default_value())
+
+	def __set__(self, obj, value):
+		'''Override the set method to allow them to set the property to an instance of the item_type instead of requiring a list to be passed in'''
+		if self.item_type in (int, long):
+			item_type = (int, long)
+		elif self.item_type in (str, unicode):
+			item_type = (str, unicode)
+		else:
+			item_type = self.item_type
+		if isinstance(value, item_type):
+			value = set(value)
+		elif value is None:  # Override to allow them to set this to 'None' to remove everything
+			value = set()
+		return super(SetProperty, self).__set__(obj, value)
+
+
 class MapProperty(Property):
-	
+
 	data_type = dict
 	type_name = 'Map'
 
@@ -673,7 +723,7 @@ class MapProperty(Property):
 		value = super(MapProperty, self).validate(value)
 		if value is not None:
 			if not isinstance(value, dict):
-				raise ValueError, 'Value must of type dict'
+				raise ValueError('Value must of type dict')
 
 		if self.item_type in (int, long):
 			item_type = (int, long)
@@ -685,7 +735,7 @@ class MapProperty(Property):
 		for key in value:
 			if not isinstance(value[key], item_type):
 				if item_type == (int, long):
-					raise ValueError, 'Values in the %s Map must all be integers.' % self.name
+					raise ValueError('Values in the %s Map must all be integers.' % self.name)
 				else:
 					raise ValueError('Values in the %s Map must all be %s instances' %
 									(self.name, self.item_type.__name__))
@@ -697,3 +747,25 @@ class MapProperty(Property):
 	def default_value(self):
 		return {}
 
+
+class JSONProperty(Property):
+
+	data_type = (dict, list, str, unicode, int, long, float, type(None))
+	type_name = 'JSON'
+
+	def __init__(self, verbose_name=None, name=None, default=None, **kwds):
+		Property.__init__(self, verbose_name, name, default=default, required=True, **kwds)
+
+	def validate(self, value):
+		import json
+		try:
+			json.dumps(value)
+		except Exception as e:
+			raise ValueError('%s in %s JSONProperty' % (str(e), self.name))
+		return value
+
+	def empty(self, value):
+		return value is None
+
+	def default_value(self):
+		return None
